////////////////////////////////////////////////////////////////////////////////
//==============================================================================
// Principal.cpp   - Compile with Borland C++ builder -
//==============================================================================
// file modified| ver.  |  by                  |         description
//--------------|-------|----------------------|--------------------------------
//   october    | 0.92  | Matthieu WEBER       | 3rd official beta release
//  16th 2003   |       |matthieu.weber@free.fr|
//--------------|-------|----------------------|--------------------------------
//   september  | 1.0   | Matthieu WEBER       | 5nd official stable release
//  08th 2009   |       |matthieu.weber@free.fr|    License : GPL 3
//--------------|-------|----------------------|--------------------------------
//   september  | 1.1   | Matthieu WEBER       | 5nd official stable release
//  20th 2011   |       |matthieu.weber@free.fr|    License : GPL 3
//==============================================================================
// Comments:
//
//  English translation of variable's names in progress...
//
////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------
#include <vcl.h>
#include <string.h>
#include <stdio.h>
#include <Registry.hpp>
#include <mmsystem.h>
#include <tapi.h>

#include <windows.h>

#pragma hdrstop
#include "Principal.h"
#include "Config.h"
#include "math.h"
#include "adCpuUsage.hpp"

#pragma package(smart_init)
#pragma link "DefautJaugeMat"
#pragma resource "*.dfm"

#define max(a, b)  (((a) > (b)) ? (a) : (b))


/***************************************************************************/
/* Variables definition                                                    */
/***************************************************************************/

TPrinc *Princ;


/***************************************************************************/
/* Functions                                                               */
/***************************************************************************/

// Users functions

//--------------------------------------------------------------------------
// Converts drive capacity into ANSISTRING
AnsiString __fastcall TPrinc::ConvertTaille(__int64 size) // Size in bytes
  {
  if (size >= 1152921504606846976) // exaoctet
    {
    return(FormatFloat("###0.# Eo", size / (float)1099511627776));
    }
  if (size >= 1125899906842624)    // petaoctet
    {
    return(FormatFloat("###0.# Po", size / (float)1099511627776));
    }
  if (size >= 1099511627776)       // teraoctet
    {
    return(FormatFloat("###0.# To", size / (float)1099511627776));
    }
  if (size >= 1073741824)          // gigaoctet
    {
    return(FormatFloat("###0.# Go", size / (float)1073741824));
    }
  if (size >= 1048576)             // megaoctet
    {
    return(FormatFloat("###0.# Mo", size / (float)1048576));
    }
  if (size >= 1024)                // kilooctet
    {
    return(FormatFloat("###0.# ko", size / (float)1024));
    }
  return(IntToStr(size) + " o");   // octet
  }

//--------------------------------------------------------------------------
// Converts seconds into hours minutes format
AnsiString __fastcall TPrinc::ConvertHeure(__int64 seconds)
  {
  short int minutes;
  short int hours;

  minutes = 60 * ((seconds / (float)3600)  - (int)(seconds / 3600));
  hours   = 24 * ((seconds / (float)86400) - (int)(seconds / 86400));
  return(FormatFloat("## h ;; ", hours) + FormatFloat("00 min. ;; ", minutes));
  }
//---------------------------------------------------------------------------

// TJaugeMat component

//---------------------------------------------------------------------------
// Composant TJaugeMat : Constructor extends from TDefautJaugeMat
__fastcall TJaugeMat::TJaugeMat(TComponent* Owner)
  : TDefautJaugeMat(Owner)
  {
  Lecteur = 'C';
  lstrcpy(Chemin, "C:\\");
  }

//---------------------------------------------------------------------------
  
// Function called when a drive is inserted or removed from the system
void __fastcall TPrinc::WMDeviceChange(TMessage &Message)
{
Nb_device_chage = true;
}

//---------------------------------------------------------------------------
// Main window constructor
__fastcall TPrinc::TPrinc(TComponent* Owner)
        : TForm(Owner)
{


}

//---------------------------------------------------------------------------
// OnDestroy event called at program's ending
void __fastcall TPrinc::FormDestroy(TObject *Sender)
  {
  // Save configuration in the system registery  
  Save_cfg();
  // free memory
  CleanUp();
  delete GaugeList;
  delete SoundCtrlList;
  }

//---------------------------------------------------------------------------
// Timer1 event occurence: when occurs: asks to refresh gauges
 void __fastcall TPrinc::Timer1Timer(TObject *Sender)
  {
  int i;

  if(((Affichage & DISPLAY_STATUS) != 0))  {StatusBar->Font->Color = clBtnText; StatusBar->Font->Name = "Small Fonts"; StatusBar->Font->Height = -8;}  // Correct a random bug (font size changes in status bar)

  if (GaugeList->Count > 0)
    {
    for(i = 0; i < GaugeList->Count; i++)
      {
      Gauge = (TJaugeMat*) GaugeList->Items[i];
      if (Gauge->Lecteur == '@')
        {
        UpdateMemory(Gauge);    // Refresh RAM
        continue;
        }
      if (Gauge->Lecteur == '#')
        {
        UpdateBattery(Gauge);   // Refresh battery status
        continue;
        }

// We don't refresh floppy disks: No infos are displayed (says dbl. click to
// refresh), until the user doubles ckick

// "tag" property is used to inform this function the kind of drive passed in
// parameter (this tag is in fact drive's icon number)
      if ((Gauge->Lecteur != 'A') && (Gauge->Lecteur != 'B'))
        {
        // We refresh all but floppy disks
        UpdateDrive(Gauge);
        }
      }
    }
  }

//---------------------------------------------------------------------------
// Timer2 event occurence=> when occurs: asks to hide the main window
void __fastcall TPrinc::Timer2Timer(TObject *Sender)
  {
  int i;

  switch (ZoneMasque)
    {
    case HIDE_RIGHT:
      Left = Screen->Width + ClientWidth - Width + 1;
    break;
    case HIDE_LEFT:
      Left = -ClientWidth - 1;
    break;
    case HIDE_TOP:
      Top = -ClientHeight - 1;
    break;
    case HIDE_BOTTOM:
      Top = Screen->Height + ClientHeight - Height + 1;
    break;
    }
  Timer2->Enabled = false;
  OnMouseMove = FormMouseMove;
  if (GaugeList->Count > 0)
    {
    for(i = 0; i < GaugeList->Count; i++)
      {
      ((TJaugeMat*) GaugeList->Items[i])->OnMouseMove = FormMouseMove;
      }
    }
  }

//---------------------------------------------------------------------------
// Displays window's explorer or file browser when double clicking on one of
// the drives
void __fastcall TPrinc::FormDblClick(TObject *Sender)

  {
  AnsiString selection;

  // Retrive the name of the drive linked with the calling gauge
  selection = ((TJaugeMat*)Sender)->Lecteur;

  if ((selection == "A")||(selection == "B"))  // If selection = floppy drive
    {
    UpdateDrive((TJaugeMat*)Sender);
    }

  if (Explorateur)
    {
    ShellExecute(Handle, "explore", strcat(selection.c_str(), ":"), NULL, NULL, SW_SHOWNORMAL);
    }
  else
    {
    ShellExecute(Handle, "open", strcat(selection.c_str(), ":"), NULL, NULL, SW_SHOWNORMAL);
    }
  }

//---------------------------------------------------------------------------
// Displays settings window if left click on a drive. - Right click: refresh drives
void __fastcall TPrinc::OnMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
int i;
// Right click
  if (Shift.Contains(ssRight))
    {
    // If the setting windows got OK button pressed
    if (Configuration->ShowModal() == mrOk)
      {
      // Save configuration in the system registery
      Save_cfg();
      // Delete old gauges and controls
      CleanUp();
      // Set up new display settings
      Affichage = Configuration->NouvAffichage;
      // Displays new jauges and controls
      DisplaysControls();
      Timer2->Enabled = bHideMainWindow;
      }
    }
    else
    // If left click
      {
      // Call Timer1 to refresh immediately datas (as user's request)
      Timer1Timer(Princ);
      }
}
//---------------------------------------------------------------------------
void __fastcall TPrinc::FormOnChange(TObject *Sender)
{
// The user has moved a sound potentiometer...  Most of theise parameters
// will not be implemented (but they are shown there, just in case...)

// Adjust audio volume
    int selection;
    selection = ((TTrackBar*)Sender)->Tag;

    switch (selection)
      {
      case 1: // Main volume is set
        {
     //   unsigned long tmp = 0;
    //    tmp = ((TTrackBar*)Sender)->Position * 655;
        //mainSetVolume(NULL, (tmp << 16) + tmp);
        if(((Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->Panels->Items[0]->Text = (AnsiString)txt_volume + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      case 2:  // Wav volume is set
        {
        unsigned long tmp = 0;
        tmp = ((TTrackBar*)Sender)->Position * 655;
        waveOutSetVolume(NULL, (tmp << 16) + tmp);
        if(((Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->Panels->Items[0]->Text = (AnsiString)txt_vol_wav + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      case 3:  // MIDI volume is set
        {
  //      unsigned long tmp = 0;
   //     tmp = ((TTrackBar*)Sender)->Position * 655;
    //    midiOutSetVolume(NULL, (tmp << 16) + tmp);

        if(((Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->Panels->Items[0]->Text = (AnsiString)txt_vol_midi + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
      }
      case 4:  // CD volume is set
        {
      //  unsigned long tmp = 0;
       // tmp = ((TTrackBar*)Sender)->Position * 655;
        //cdSetVolume(NULL, (tmp << 16) + tmp);
        if(((Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->Panels->Items[0]->Text = (AnsiString)txt_vol_cd + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      case 5: // Auxiliary input volume is set
        {
     //   unsigned long tmp = 0;
     //   tmp = ((TTrackBar*)Sender)->Position * 655;
     //   auxSetVolume(NULL, (tmp << 16) + tmp);

        if(((Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->Panels->Items[0]->Text = (AnsiString)txt_vol_aux + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      case 6:  // Microphone volume is set
        {
    //    unsigned long tmp = 0;
   //     tmp = ((TTrackBar*)Sender)->Position * 655;
        //micSetVolume(NULL, (tmp << 16) + tmp);
        if(((Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
          {
          StatusBar->Panels->Items[0]->Text = (AnsiString)txt_vol_micro + " " + \
          (AnsiString)((TTrackBar*)Sender)->Position + " " +(AnsiString) "%";
          }
        break;
        }
      }
      // Stop and restart the 1 second Timer attached to the status bar in
      // order to see volume change exactly during one second after the last
      // user's volume change

      StatusBarTimer->Enabled = false;
      StatusBarTimer->Enabled = true;
}

//---------------------------------------------------------------------------
// OnMouseMove event of the main windows, to show the hidden window
void __fastcall TPrinc::FormMouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y)
{
  UnHide(bHideMainWindow);
}

//---------------------------------------------------------------------------
// Add gauges or controls and let them be displayed according to the user
// settings. This function is called at the begining of the application and
// every times we need to rescan and display the gauges and controls.
void __fastcall TPrinc::DisplaysControls()
{
  // Set main window's size to the smallest value
  MainWindowHeight = 27;
  char Temp[4];
  DWORD Lecteurs;
  unsigned int NumLettre = 0;

  // Specifiy main window's horizontal size (+5 is the external bevel)
  Width = JAUGE_WIDTH + 5;


  // If showing RAM gauge
  if ((Affichage & DISPLAY_MEMORY) != 0)
    {
    AddGauge('@', 5);
    }

  // If showing battery charge gauge
  if ((Affichage & DISPLAY_BATTERY) != 0)
    {// 0 => Battery icon was inserted manually in the icon list "ImageList1"
    AddGauge('#', 0);
    }

  // Retrive system's drive handlers then display them
  Lecteurs = GetLogicalDrives();
  lstrcpy(Temp, "A:\\");
  while (Lecteurs > 0)
    {
    if (Lecteurs & 1 == 1)
      {
      // Retrive data mask
      Temp[0]= NumLettre + 'A';
      switch(GetDriveType(Temp))
        {
        case 0:
          MessageBox(Handle, Temp, txt_unknown_drive_type.c_str(), MB_OK);
          break;
        case 1:
          // Unable to find the drive path
          break;
        case DRIVE_REMOVABLE:
          if ((Affichage & DISPLAY_REMOVABLE_DRV) != 0)
            AddGauge(Temp[0], 1);
          break;
        case DRIVE_FIXED:
          if ((Affichage & DISPLAY_FIXED_DRV) != 0)
            AddGauge(Temp[0], 2);
          break;
        case DRIVE_REMOTE:
          if ((Affichage & DISPLAY_NET_DRV) != 0)
            AddGauge(Temp[0], 3);
          break;
        case DRIVE_CDROM:
          if ((Affichage & DISPLAY_CDROM) != 0)
            AddGauge(Temp[0], 4);
          break;
        case DRIVE_RAMDISK:
          if ((Affichage & DISPLAY_RAMDISK) != 0)
            AddGauge(Temp[0], 5);
          break;
        }
      }
      NumLettre++;
      Lecteurs = Lecteurs >> 1;
    }

// Display (optionnal) audio controls

// Main volume control
  if ((Affichage & DISPLAY_MAIN_VOL) != 0)
    {
    AddSoundControl("Vol.", 1);
    }
// Wav volume control
  if ((Affichage & DISPLAY_WAV_VOL) != 0)
    {
    AddSoundControl("Wav", 2);
    }
// MIDI volume control
  if ((Affichage & DISPLAY_MIDI_VOL) != 0)
    {
    AddSoundControl("MIDI", 3);
    }
// CD volume control
  if ((Affichage & DISPLAY_CD_VOL) != 0)
    {
    AddSoundControl("CD", 4);
    }
// Auxiliary input volume control
  if ((Affichage & DISPLAY_AUX_VOL) != 0)
    {
    AddSoundControl("Aux.", 5);
    }
// Microphone volume control
  if ((Affichage & DISPLAY_MIC_VOL) != 0)
    {
    AddSoundControl("Mic.", 6);
    }

// Display status bar (optionnal)

  if ((Affichage & DISPLAY_STATUS) !=0)
    {
    StatusBar = new TStatusBar("");
    StatusBar->Parent = Princ;
    StatusBar->Font->Name = "Small Fonts";
    StatusBar->Font->Height = -8;
    StatusBar->Font->Color = clBtnText;
    StatusBar->Height = 15;
    StatusBar->SimplePanel = false; // more than one panels are created...
    StatusBar->Panels->Add();       // .. One for common messages
    StatusBar->Panels->Add();       // .. The second one for CPU usage
    StatusBar->Panels->Items[0]->Width = 75;
// Adjust main window's vertical size to display status bar
    MainWindowHeight += StatusBar->Height;
    }
  MainWindowHeight -= 4;
  Height = MainWindowHeight;

  // Call Timer1Timer to display immediately the (first collected) datas
  Timer1Timer(Princ);
}

//---------------------------------------------------------------------------
// Adds gauges after thoses already displayed
void __fastcall TPrinc::AddGauge(char Lecteur, unsigned char IndiceIco)
  {
  Graphics::TBitmap* temp;
  char NomVol[64];

  Gauge= new TJaugeMat(Princ);
  GaugeList->Add(Gauge);
  Gauge->Parent = Princ;
  Gauge->Font->Name = "Small Fonts";
  Gauge->Font->Height = -8;
  Gauge->Left = 0;
  Gauge->Top = (GaugeList->Count - 1) * (JAUGE_HEIGHT);
  Gauge->Height = JAUGE_HEIGHT;
  Gauge->Width = JAUGE_WIDTH;

  // Adjust main window's vertical size to add the new control
  MainWindowHeight += Gauge->Height;

  // Here, the "tag" property is used to send the kind of drive in parameters
  // (via his icon ID) to the function "UpdateDrive"
  Gauge->Tag = IndiceIco;

  if (Lecteur == '@')
    Gauge->Label = txt_memoire;               // If current gauge is RAM
  else
    if ((Lecteur != '@') && (Lecteur != '#')) // If current gauge is a drive
      {
      Gauge->Chemin[0] = Lecteur;             // displays the path
      Gauge->OnDblClick = FormDblClick;       // double click enabled
      }

    Gauge->Lecteur = Lecteur;                 // In all cases...
    Gauge->Pourcentage = true;
    Gauge->OnMouseDown = OnMouseDown;         // ...Right click enabled to acess config window

    if ((Lecteur != '@') && (Lecteur != '#'))
      {
      Gauge->Pourcentage = false;  // Hide "%" if there is no disk in drive
      // Displays a text asking user to click to display the floppy drive's infos
      Gauge->Label = AnsiString("( ") + AnsiString(Gauge->Lecteur) + AnsiString(" ) ") + txt_rafraich;
      }

    // If shading display (texture display)
    if (Texture)
      {
      Graphics::TBitmap *temp = new Graphics::TBitmap();
      // Shading choice
      ImageList2->GetBitmap(Type_Texture, temp);
      if (InvertDisp)
        {
        // We invert the bitmap to show an inverted jauge (not implemented yet)
        ;
        }
      Gauge->Bitmap = temp;
      delete temp;
      }

    Gauge->Texture = Texture;
    ImageList1->GetIcon(IndiceIco, Gauge->Icone);
    }
//---------------------------------------------------------------------------
// Adds others controls after thoses already displayed
void __fastcall TPrinc::AddSoundControl(char* Controle, unsigned char index)
  {
  Volume = new TTrackBar(Princ);
  SoundCtrlList->Add(Volume);
  Volume->Parent = Princ;
  Volume->Left = 0;
  Volume->Top = (GaugeList->Count) * JAUGE_HEIGHT + \
        (SoundCtrlList->Count - 1) * (VOLUME_CTRL_HEIGHT + 2) + 2;
  Volume->Tag = index;

  // Volume control
   Volume->Left = 20;
   Volume->Height = 15;
   Volume->Width = 95;
   Volume->ThumbLength = 10;
   Volume->Max = 100;
   Volume->Frequency = 10;
   Volume->LineSize = 5;
   Volume->PageSize = 8;
   Volume->TickMarks = Comctrls::tmBoth;
   Volume->TickStyle = tsNone;
   Volume->OnChange = FormOnChange;  // Attaching on chnage => volume event

   VolCtrlBevel = new TBevel(Princ);
   VolCtrlBevel->Shape =  bsFrame;
   VolCtrlBevel->Parent = Princ;
   VolCtrlBevel->Left = 0;
   VolCtrlBevel->Top = (GaugeList->Count) * JAUGE_HEIGHT + \
        (SoundCtrlList->Count - 1) * (VOLUME_CTRL_HEIGHT + 2);
   VolCtrlBevel->Height = 19;
   VolCtrlBevel->Width = JAUGE_WIDTH;

   VolTxt = new TLabel(Princ);
   VolTxt->Caption = "Vol.";
   VolTxt->Font->Name = "Small Fonts";
   VolTxt->Font->Height = -8;
   VolTxt->Parent = Princ;
   VolTxt->Left = 3;
   VolTxt->Top = 5 + (GaugeList->Count) * JAUGE_HEIGHT + \
        (SoundCtrlList->Count - 1) * (VOLUME_CTRL_HEIGHT + 2);

 /*  SelectAudio = new  TSpeedButton(Princ);
   SelectAudio->Parent = Princ;
   SelectAudio->Height = 9;
   SelectAudio->Width = 9;
   SelectAudio->Left = 107;
   SelectAudio->Top = 5 + (GaugeList->Count) * JAUGE_HEIGHT + \
        (SoundCtrlList->Count - 1) * (VOLUME_CTRL_HEIGHT + 2);
                                                                */
  // Adjust vertical window size to show the new control
  MainWindowHeight += VolCtrlBevel->Height-2;
    }
//---------------------------------------------------------------------------
// Delete all controls and gauges, free the memory
void __fastcall TPrinc::CleanUp()
  {
  while(GaugeList->Count > 0)
    {
    Gauge = (TJaugeMat*) GaugeList->First();
    delete Gauge;
    GaugeList->Delete(0);
    }

  while(SoundCtrlList->Count > 0)
    {
//    Volume = (TTrackBar*) SoundCtrlList->First();
    delete Volume;
    delete VolTxt;
    delete VolCtrlBevel;
//    delete SelectAudio;
    
    SoundCtrlList->Delete(0);
    }

    if(((Affichage & DISPLAY_STATUS) != 0)) //    If (StatusBar != NULL)
      {
      delete StatusBar;
      }
}
//---------------------------------------------------------------------------
// Retrive system drives icons and put them in the component (ImageList1)
void __fastcall TPrinc::GetIcons()
  {
  char Fich[MAX_PATH];
  TIcon *Ico;

  Ico = new TIcon;
  GetSystemDirectory(Fich, MAX_PATH);
  lstrcat(Fich, "\\Shell32.dll");
  /*
  Ico.Handle:= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(4));  // 0: Closed folder
  ImageList1.AddIcon(Ico);
  Ico.Handle:= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(5));  // 1: Open folder
  ImageList1.AddIcon(Ico);
  */
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(7));  // 2: Floppy
  ImageList1->AddIcon(Ico);
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(9));  // 3: Hard drive
  ImageList1->AddIcon(Ico);
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(10)); // 4: Network drive
  ImageList1->AddIcon(Ico);
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(12)); // 5: CDROM DVD
  ImageList1->AddIcon(Ico);
  Ico->Handle= LoadIcon(LoadLibrary(Fich), MAKEINTRESOURCE(13)); // 6: RAM
  ImageList1->AddIcon(Ico);
  delete Ico;
  }

//---------------------------------------------------------------------------
// Refresh the RAM gauge
void __fastcall TPrinc::UpdateMemory(TJaugeMat *Jauge)
  {
  MEMORYSTATUSEX EtatMem;
  unsigned char i;
  bool Trouve;

  EtatMem.dwLength = sizeof (EtatMem);
  GlobalMemoryStatusEx (&EtatMem);

  Gauge->Position = EtatMem.dwMemoryLoad;
  Gauge->Max = 100;
  Gauge->Libre = ConvertTaille(EtatMem.ullAvailPhys);
  Gauge->Capacite = ConvertTaille(EtatMem.ullTotalPhys);

  if (!Texture)
    {
    if (!Uniform)
      {
      i = 0;
      Trouve = false;
      while (!Trouve && (i < (MAX_THRESHOLDS - NbSeuils)))
        {
        if (Jauge->Position < ValSeuil[i])
          {
          Trouve = true;
          Gauge->CoulBarre = Couleur[i];
          }
        else
          {
          i++;
          }
        }
      if (!Trouve)
        Gauge->CoulBarre = Couleur[i];
      }
    else
      Gauge->CoulBarre = Couleur[4];
    }
  }
  
//----------------------------------------------------------------------------
// Refresh the battery charge gauge
void __fastcall TPrinc::UpdateBattery(TJaugeMat *Jauge)
{
SYSTEM_POWER_STATUS PowerStatus;
unsigned char i;
bool Trouve;

// Get system power infos
GetSystemPowerStatus(&PowerStatus);

// AC supply connected and powered
if (PowerStatus.ACLineStatus == 1)
  {
  if (PowerStatus.BatteryFlag & 0x08)  Gauge->Label = txt_batt_charge;  else  Gauge->Label = txt_batteries;
  }
else
  {
  if (PowerStatus.BatteryFlag & 0x04)  Gauge->Label = txt_batt_critique; else Gauge->Label = txt_batt_debit;
  }

if (PowerStatus.BatteryFlag & 0x80)  Gauge->Label = txt_batt_non;
if (PowerStatus.BatteryFlag == 0xFF) Gauge->Label = txt_batt_erreur;

if (PowerStatus.BatteryLifePercent != 255)  // If we can perform a measurement...
  {
  Gauge->Pourcentage = true; // If a battery is installed display the "%" sign
  Gauge->Position    = PowerStatus.BatteryLifePercent;
  Gauge->Max         = 100;

  if (PowerStatus.ACLineStatus == 0)  // Display battery actual life time / actual full life time
    {
    Gauge->Libre       = ConvertHeure(PowerStatus.BatteryLifeTime);
    Gauge->Capacite    = ConvertHeure(PowerStatus.BatteryLifeTime * (100.0 / float(PowerStatus.BatteryLifePercent)));
    }
  else
    {
    Gauge->Libre       = "-- h -- min.";   // Cannot get battery data
    Gauge->Capacite    = "-- h -- min.";
    }
  }
else
  {
  Gauge->Position    = 0;  // Unable to make measurement, no battery ?
  Gauge->Pourcentage = false; // If no battery installed hide the "%" sign
  }

if (!Texture)
  {
  if (!Uniform)
    {
    i = 0;
    Trouve = false;
    while (!Trouve && (i < (MAX_THRESHOLDS - NbSeuils)))
      {
      if (Jauge->Position < ValSeuil[i])
        {
        Trouve = true;
        Jauge->CoulBarre = Couleur[i];
        }
      else
        {
        i++;
        }
      }
    if (!Trouve)  Jauge->CoulBarre = Couleur[i];
    }
  else Jauge->CoulBarre = Couleur[4];
  }
}

//----------------------------------------------------------------------------
// Gets drive information from the jauge passed in parameters this function is
// called every "MAJNORM" seconds and one time at program startup
void __fastcall TPrinc::UpdateDrive(TJaugeMat *Gauge)
{
__int64 Bidon, TailTot, TailLib;
unsigned char i;
bool Trouve = 0;
char NomVol[13];
NomVol[0] = '\0';
static AnsiString tmp;
try
  {
  if (GetVolumeInformation(Gauge->Chemin, NomVol, 64, NULL, NULL, NULL, NULL, 0))
    {
    tmp = AnsiString("( ") + AnsiString(Gauge->Lecteur) + AnsiString(" ) ") + AnsiString(NomVol);
    while (Canvas->TextExtent(tmp).cx > Gauge->Width)
      {
      tmp.SetLength(tmp.Length()-1);
      Trouve = 1;
      }

    if (Trouve) tmp[tmp.Length()] = '';

    Gauge->Label = tmp;
    }
    else
      {
      // If we where unable to retreive label name
      Gauge->Label = AnsiString("( ") + AnsiString(Gauge->Lecteur) + AnsiString(" ) ");
      }
    if (Sysutils::GetDiskFreeSpaceEx(Gauge->Chemin, Bidon, TailTot, &TailLib))
      {
      Gauge->Max = 100; // In all cases
      if (Gauge->Tag != TAG_CD) // If drive is not a CDROM / DVD
        {
        if(InvertDisp)
          {
          // If we choose to show the space occuped
          Gauge->Position = 100 - (100 * ((float) TailLib / TailTot));
          }
          else
          {
          // If we choose to show the free space
          Gauge->Position = 100 * ((float) TailLib / TailTot);
          }


        Gauge->Libre = ConvertTaille(TailLib);
        Gauge->Capacite = ConvertTaille(TailTot);
        }
      else
        {
        Gauge->Capacite = ConvertTaille(TailLib);
        if (TailTot > MAX_CD_700)  // If a DVD and lesser than MAX_DVD value
          {
          Gauge->Libre = ConvertTaille(MAX_DVD - TailTot); //  (<= MAX_DVD)
          if (TailTot > MAX_DVD) // HUGE  stuff !! => the gauge is set to the
                                                  // capacity of the disk
            {
            Gauge->Capacite = ConvertTaille(TailTot);
            Gauge->Position = 0;
            Gauge->Libre = 0;
            }
          else
            {
            Gauge->Capacite = ConvertTaille(MAX_DVD);
            Gauge->Position = 100 * (1-((float) TailTot / MAX_DVD));
            }
          }
        else
          {
            if (TailTot > MAX_CD_655) // May be a 700 Mo CD: MAX_CD_700 Mo
            {
            Gauge->Libre = ConvertTaille(MAX_CD_700 - TailTot); // (<= MAX_CD_700 Mo)
            Gauge->Capacite = ConvertTaille(MAX_CD_700);
            Gauge->Position = 100 * (1-((float) TailTot / MAX_CD_700));
            }
          else                      // May be a 650 Mo CD +/- MAX_CD_655 Mo
            {
            Gauge->Libre = ConvertTaille(MAX_CD_655 - TailTot); // (<= MAX_CD_655Mo)
            Gauge->Capacite = ConvertTaille(MAX_CD_655);
            Gauge->Position = 100 * (1-((float) TailTot / MAX_CD_655));
            }
          }
        }

      if (!Texture)
        {
        if (!Uniform)
          {
          i = 0;
          Trouve = false;
          while (!Trouve && (i < (MAX_THRESHOLDS - NbSeuils)))
            {
            if (Gauge->Position < ValSeuil[i])
              {
              Trouve = true;
              Gauge->CoulBarre = Couleur[i];
              }
            else
              {
              i++;
              }
            }
          if (!Trouve)
            Gauge->CoulBarre = Couleur[i];
          }
        else
          {
          Gauge->CoulBarre = Couleur[4];
          }

        }
      Gauge->Pourcentage = true; // Displays the "%" sign if non empty drive
      }
  // If GetDiskFreeSpaceEx fails, drive should be empty
    else
      {
      Gauge->Position = 0;
      Gauge->Max = 100; // In all cases
      Gauge->Libre = "";
      Gauge->Capacite = "";
      Gauge->Pourcentage = false;  // Hide the "%" sign if empty drive
      Gauge->Label = AnsiString("( ") + AnsiString(Gauge->Lecteur) + AnsiString(" ) ") + txt_lect_vide;
      }
  }
catch (Exception &exception)  // Discard error while getting drive's informations
  {
  ;
  }
}

//---------------------------------------------------------------------------
// Displays the window near the edge it was beafore beeing hidden
void __fastcall TPrinc::UnHide(bool Remasquer)
  {
  int i;

  switch (ZoneMasque)
    {
    case HIDE_RIGHT:
      Left = Screen->Width - Width;
      Left = Left;
    break;
    case HIDE_LEFT:
      Left = 0;
    break;
    case HIDE_TOP:
      Top = 0;
    break;
    case HIDE_BOTTOM:
      Top = Screen->Height - Height;
    break;
    }
  Timer2->Enabled = Remasquer;
  OnMouseMove = NULL;
  if (GaugeList->Count > 0)
    {
    for(i = 0; i < GaugeList->Count; i++)
      ((TJaugeMat*) GaugeList->Items[i])->OnMouseMove = NULL;
    }
  }

//----------------------------------------------------------------------------
// Saves settings in the system registery
void __fastcall TPrinc::Save_cfg(void)
{
  TRegistry *Reg;

  // Init the registery
  Reg = new  TRegistry;

  // Open 'current user' key
  Reg->RootKey = HKEY_CURRENT_USER;

  // Open/create 'Espace_Libre' key
  if (Reg->OpenKey(REG_VAL_SOFTW_PATH, true))
  {
    // Write values in 'Espace_Libre' key
    Reg->WriteBool(REG_VAL_ON_TOP, FormStyle == fsStayOnTop);
    Reg->WriteInteger(REG_VAL_LEFT, Left);
    Reg->WriteInteger(REG_VAL_TOP, Top);
    Reg->WriteInteger(REG_VAL_DISPLAY_CTRLS, Affichage);
    Reg->WriteBool(REG_VAL_TEXTURE, Texture);
    Reg->WriteBool(REG_VAL_UNIFORM, Uniform);
    Reg->WriteInteger(REG_VAL_NSEUILS, NbSeuils);
    Reg->WriteInteger(REG_VAL_MAJNORM, Timer1->Interval);
    Reg->WriteBool(REG_VAL_MASQUER, bHideMainWindow);
    Reg->WriteInteger(REG_VAL_DURVIS, Timer2->Interval);
    Reg->WriteInteger(REG_VAL_ZONMASQ, ZoneMasque);
    Reg->WriteInteger(REG_VAL_COUL0, Couleur[0]);
    Reg->WriteInteger(REG_VAL_SEUIL0, ValSeuil[0]);
    Reg->WriteInteger(REG_VAL_COUL1, Couleur[1]);
    Reg->WriteInteger(REG_VAL_SEUIL1, ValSeuil[1]);
    Reg->WriteInteger(REG_VAL_COUL2, Couleur[2]);
    Reg->WriteInteger(REG_VAL_SEUIL2, ValSeuil[2]);
    Reg->WriteInteger(REG_VAL_COUL3, Couleur[3]);
    Reg->WriteInteger(REG_VAL_COUL4, Couleur[4]);
    Reg->WriteInteger(REG_VAL_TYPE_TEXTURE, Type_Texture);
    Reg->WriteBool(REG_VAL_EXPLORATEUR, Explorateur);
    Reg->WriteBool(REG_VAL_AUTODEMARRAGE, Autodemarrage);
    }
  delete Reg;

  // Create or remove key used to auto start the application
  TRegistry *Reg2 = new TRegistry();

  // Write in  HKEY_CURRENT_USER
  Reg2->RootKey = HKEY_CURRENT_USER;
  if (Reg2->OpenKey("\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", false))
    {
    if (Autodemarrage == true)
      {
      AnsiString temp = GetCurrentDir();
      if (temp[temp.Length()] != '\\')
        {
        temp = temp + "\\";
        }
      Reg2->WriteString("Espace_Libre.exe", temp + "Espace_Libre.exe");
      }
    else
      {
      Reg2->DeleteValue("Espace_Libre.exe");
      }
    }
  delete Reg2;
}

//---------------------------------------------------------------------------
// Reads the system registery to gets the settings of the application
// Also read the Espace_Libre.ini file (if exists) and therefore set the foreigns
// languages (other than french). Sets dfaults values if the key was not created yet
// The ini file is never written by the application.
void __fastcall TPrinc::Load_cfg(void)
  {
  TRegistry *Reg;

  // Init the system registery
  Reg = new  TRegistry;

  // Open 'current user' key
  Reg->RootKey = HKEY_CURRENT_USER;

  // Open/create 'Espace_Libre' key
  if (Reg->OpenKey(REG_VAL_SOFTW_PATH, false))
    {
    if (Reg->ValueExists(REG_VAL_ON_TOP))
      if (Reg->ReadBool(REG_VAL_ON_TOP))
        FormStyle = fsStayOnTop;
      else
        FormStyle = fsNormal;
    else
      FormStyle = REG_VAL_DEF_ON_TOP;

    if (Reg->ValueExists(REG_VAL_LEFT))
      {
      Left = Reg->ReadInteger(REG_VAL_LEFT);  // reads the value (may be altered just after if the applicaton is out of the display)

      if (Left >= Screen->DesktopWidth) // Last time, if Espace_Libre was hidden due to a resolution change... (we allows a extra in case of the autohide feature is enabled!!)
        Left = Screen->DesktopWidth - Width;  // ...We move Espace_Libre to the right eidge of the screen to keep control !
      }
    else
      Left = Screen->DesktopWidth - Width; // Fail to read config register (first launch?) => ...We move Espace_Libre to the right eidge of the screen

    if (Reg->ValueExists(REG_VAL_TOP))
      {
      Top = Reg->ReadInteger(REG_VAL_TOP); // reads the value (may be altered just after if the applicaton is out of the display)

      if (Top < -10) // // Last time, if Espace_Libre is hidden due to a resolution change... (Top <= 0 => begins to be hidden
        Top = 0;  // ...We move Espace_Libre to the upper eidge of the screen to keep control !
      }
    else
      Top = Screen->DesktopHeight - Height; // Fail to read config register (first launch?) => ...We move Espace_Libre to the upper eidge of the screen

    if (Reg->ValueExists(REG_VAL_DISPLAY_CTRLS))
      Affichage = Reg->ReadInteger(REG_VAL_DISPLAY_CTRLS);
    else
      Affichage = REG_VAL_DEF_DISPLAY_CTRLS;

    if (Reg->ValueExists(REG_VAL_TEXTURE))
      Texture = Reg->ReadBool(REG_VAL_TEXTURE);
    else
      Texture = REG_VAL_DEF_SHADING_ON;

    if (Reg->ValueExists(REG_VAL_UNIFORM))
      Uniform = Reg->ReadBool(REG_VAL_UNIFORM);
    else
      Uniform = REG_VAL_DEF_NO_THRESHOLDS;

    if (Reg->ValueExists(REG_VAL_NSEUILS))
      NbSeuils = Reg->ReadInteger(REG_VAL_NSEUILS);
    else
      NbSeuils = REG_VAL_DEF_NB_THRESHOLDS;

    if (Reg->ValueExists(REG_VAL_COUL0))
      Couleur[0] = (TColor) Reg->ReadInteger(REG_VAL_COUL0);
    else
      Couleur[0] = REG_VAL_DEF_COLOR0;

    if (Reg->ValueExists(REG_VAL_SEUIL0))
      ValSeuil[0] = Reg->ReadInteger(REG_VAL_SEUIL0);
    else
      ValSeuil[0] = REG_VAL_DEF_THRESHOLD0;

    if (Reg->ValueExists(REG_VAL_COUL1))
      Couleur[1] = (TColor) Reg->ReadInteger(REG_VAL_COUL1);
    else
      Couleur[1] = REG_VAL_DEF_COLOR1;

    if (Reg->ValueExists(REG_VAL_SEUIL1))
      ValSeuil[1] = Reg->ReadInteger(REG_VAL_SEUIL1);
    else
      ValSeuil[1] = REG_VAL_DEF_THRESHOLD1;

    if (Reg->ValueExists(REG_VAL_COUL2))
      Couleur[2] = (TColor) Reg->ReadInteger(REG_VAL_COUL2);
    else
      Couleur[2] = REG_VAL_DEF_COLOR2;

    if (Reg->ValueExists(REG_VAL_SEUIL2))
      ValSeuil[2] = Reg->ReadInteger(REG_VAL_SEUIL2);
    else
      ValSeuil[2] = REG_VAL_DEF_THRESHOLD2;

    if (Reg->ValueExists(REG_VAL_COUL3))
      Couleur[3] = (TColor) Reg->ReadInteger(REG_VAL_COUL3);
    else
      Couleur[4] = REG_VAL_DEF_COLOR3;

    if (Reg->ValueExists(REG_VAL_COUL4))
      Couleur[4] = (TColor) Reg->ReadInteger(REG_VAL_COUL4);
    else
      Couleur[4] = REG_VAL_DEF_COLOR4;

    if (Reg->ValueExists(REG_VAL_TYPE_TEXTURE))
      Type_Texture = Reg->ReadInteger(REG_VAL_TYPE_TEXTURE);
    else
      Type_Texture = REG_VAL_DEF_SHADING_TYPE;

    if (Reg->ValueExists(REG_VAL_MAJNORM))
      Timer1->Interval = Reg->ReadInteger(REG_VAL_MAJNORM);
    else
      Timer1->Interval = REG_VAL_DEF_NORMAL_REFRESH_RATE;

    if (Reg->ValueExists(REG_VAL_MASQUER))
      bHideMainWindow = Reg->ReadBool(REG_VAL_MASQUER);
    else
      bHideMainWindow = REG_VAL_DEF_AUTO_HIDE;

    if (Reg->ValueExists(REG_VAL_DURVIS))
      Timer2->Interval = Reg->ReadInteger(REG_VAL_DURVIS);
    else
      Timer2->Interval = REG_VAL_DEF_VISIBILITY_TIME;

    if (Reg->ValueExists(REG_VAL_ZONMASQ))
      ZoneMasque = Reg->ReadInteger(REG_VAL_ZONMASQ);
    else
      ZoneMasque = REG_VAL_DEF_HIDDING_ZONE;

    if (Reg->ValueExists(REG_VAL_EXPLORATEUR))
      Explorateur = Reg->ReadInteger(REG_VAL_EXPLORATEUR);
    else
      Explorateur = REG_VAL_DEF_EXPLORER_BROWSER;

    if (Reg->ValueExists(REG_VAL_AUTODEMARRAGE))
      Autodemarrage = Reg->ReadInteger(REG_VAL_AUTODEMARRAGE);
    else
      Autodemarrage = REG_VAL_DEF_AUTO_LAUNCH;
    }
  else // If the key is not created yet
    {
    FormStyle        = REG_VAL_DEF_ON_TOP;
    Left             = (Screen->Width - Width) / 2;
    Top              = (Screen->Height - Height) / 2;
    Affichage        = REG_VAL_DEF_DISPLAY_CTRLS;
    Texture          = REG_VAL_DEF_SHADING_ON;
    NbSeuils         = REG_VAL_DEF_NB_THRESHOLDS;
    Couleur[0]       = REG_VAL_DEF_COLOR0;
    ValSeuil[0]      = REG_VAL_DEF_THRESHOLD0;
    Couleur[1]       = REG_VAL_DEF_COLOR1;
    ValSeuil[1]      = REG_VAL_DEF_THRESHOLD1;
    Couleur[2]       = REG_VAL_DEF_COLOR2;
    ValSeuil[2]      = REG_VAL_DEF_THRESHOLD2;
    Couleur[3]       = REG_VAL_DEF_COLOR3;
    Couleur[4]       = REG_VAL_DEF_COLOR4;
    Timer1->Interval = REG_VAL_DEF_NORMAL_REFRESH_RATE;
    bHideMainWindow  = REG_VAL_DEF_AUTO_HIDE;
    Timer2->Interval = REG_VAL_DEF_VISIBILITY_TIME;
    ZoneMasque       = REG_VAL_DEF_HIDDING_ZONE;
    Explorateur      = REG_VAL_DEF_EXPLORER_BROWSER;
    Autodemarrage    = REG_VAL_DEF_AUTO_LAUNCH;
    }
  Timer2->Enabled = bHideMainWindow;
  delete Reg;

// Gets the language string codes

// Init the inifile component
TIniFile* ini;

ini = new TIniFile(ChangeFileExt( Application->ExeName, ".INI" ) );

// Windows error
txt_unknown_drive_type = ini->ReadString("Language", "\"Unable to determine the drive type\"", DEF_INI_VAL_TXT_BATTERY);

// Main window strings
txt_batteries          = ini->ReadString("Language", "\"Battery\"", DEF_INI_VAL_TXT_BATTERY);
txt_memoire            = ini->ReadString("Language", "\"Memory\"", DEF_INI_VAL_TXT_MEMORY);
txt_rafraich           = ini->ReadString("Language", "\"Db. click to refresh...\"", DEF_INI_VAL_TXT_NEED_REFRESH);
txt_lect_vide          = ini->ReadString("Language", "\"No disc in drive\"", DEF_INI_VAL_TXT_NO_DISK);
txt_batt_non           = ini->ReadString("Language", "\"No battery set\"", DEF_INI_VAL_TXT_NO_BATTERY);
txt_batt_debit         = ini->ReadString("Language", "\"Using battery...\"", DEF_INI_VAL_TXT_BATTERY_POWERED);
txt_batt_faible        = ini->ReadString("Language", "\"Low battery!\"", DEF_INI_VAL_TXT_LOW_BATTERY);
txt_batt_critique      = ini->ReadString("Language", "\"Low batt critical!\"", DEF_INI_VAL_TXT_BATTERY_CRITICAL);
txt_batt_charge        =  ini->ReadString("Language", "\"Battery charging...\"", DEF_INI_VAL_TXT_BATTERY_CHARGING);
txt_batt_erreur        = ini->ReadString("Language", "\"- Battery satus error! -\"", DEF_INI_VAL_TXT_BATTERY_STAT_ERROR);
txt_volume             = ini->ReadString("Language", "\"MAIN volume\"", DEF_INI_VAL_TXT_MAIN_VOL);
txt_vol_midi           = ini->ReadString("Language", "\"MIDI volume\"", DEF_INI_VAL_TXT_MIDI_VOL);
txt_vol_aux            = ini->ReadString("Language", "\"AUX. volume\"", DEF_INI_VAL_TXT_AUX_VOL);
txt_vol_micro          = ini->ReadString("Language", "\"MIC. volume\"", DEF_INI_VAL_TXT_MIC_VOL);
txt_vol_wav            = ini->ReadString("Language", "\"WAV volume\"", DEF_INI_VAL_TXT_WAV_VOL);
txt_vol_cd             = ini->ReadString("Language", "\"CD volume \"", DEF_INI_VAL_TXT_CD_VOL);
txt_mem_10_pourcent    = ini->ReadString("Language", "\"Mem.\"", DEF_INI_VAL_TXT_MEM_10_PERCENT);
txt_batt_10_pourcent   = ini->ReadString("Language", "\"BATT.\"", DEF_INI_VAL_TXT_BATT_10_PERCENT);
txt_suffixe_X_pourcent = ini->ReadString("Language", "\"< 2 %\"", DEF_INI_VAL_TXT_SUFFIX_XX_PERCENT);

delete ini;

}

//----------------------------------------------------------------------------
void __fastcall TPrinc::Timer1s(TObject *Sender)
// Used to displays some infos in the status bar and to adjust volume controls
// this timer is called every seconds
{
 unsigned long tmp, tmp1;
 int i, cpu;
 bool bDisplayInfosInStatusBar = 0;
 TTrackBar *potentiometer = new TTrackBar(Princ);

 //----------

  // Get sound volume and then adjust volume controls
  if (SoundCtrlList->Count > 0)   // Limited for now to only WAV setting !!
    {
	try
      {
      for(i = 0; i < SoundCtrlList->Count; i++)
        {
        Volume = (TTrackBar*)SoundCtrlList->Items[i];
        waveOutGetVolume(NULL, &tmp);
        tmp1 = tmp  >> 16;    // Left channel
        Volume->Position = max(tmp1, tmp - (tmp1 << 16))/655;
        }
      }
    catch (Exception &exception)  // Discard error while getting drive's informations
      {
      ;
      }
    }


  try
    {
    if(((Affichage & DISPLAY_STATUS) != 0))  // if (StatusBar != NULL)
      {
      CollectCPUData(); // Get the data for all processors

      // Displays CPU load each seconds
      cpu = 0;
      for (i = 0; i < GetCPUCount()-1; i++) cpu+=GetCPUUsage(i)*100;  // calculate total cpu usage from CPU(s) installed on this computer

      cpu/= GetCPUCount()-1;

      if (cpu > 100)  cpu = 100;    // avoid short spurious results

      StatusBar->Panels->Items[1]->Text = "CPU " + AnsiString(cpu) + "%";
      StatusBar->Panels->Items[0]->Text = "";
      }
    }
  catch (Exception &exception)  // Discard error while getting drive's informations
    {
    ;
    }


  if (GaugeList->Count > 0)
    {
    for(i = 0; i < GaugeList->Count; i++)
      {
      Gauge = (TJaugeMat*) GaugeList->Items[i];

      // Set a visual alarm if Drive(s), battery or memory fall below a value
      if ((Gauge->Position <= 10) && (Gauge->Capacite.Length() != 0) && (Gauge->Tag != TAG_CD))
        {
        // Set a visual alarm if current gauge is the battery voltage and is below 10 %
        if (Gauge->Lecteur == '#')
          {
          bDisplayInfosInStatusBar = true;
          if(((Affichage & DISPLAY_STATUS) != 0))  StatusBar->Panels->Items[0]->Text = StatusBar->Panels->Items[0]->Text + txt_batt_10_pourcent;
          continue;
          }

        // Set a visual alarm if current gauge is the RAM and is below 10 %
       if (Gauge->Lecteur == '@')
          {
          bDisplayInfosInStatusBar = true;
          if(((Affichage & DISPLAY_STATUS) != 0))  StatusBar->Panels->Items[0]->Text = StatusBar->Panels->Items[0]->Text + txt_mem_10_pourcent;
          continue;
          }

        // Set a visual alarm if current gauge is a drive and is below 2 % of free space
        if (Gauge->Position <= 2)
          {
          bDisplayInfosInStatusBar = true;
          if(((Affichage & DISPLAY_STATUS) != 0))  StatusBar->Panels->Items[0]->Text = StatusBar->Panels->Items[0]->Text +
              " " + (AnsiString)Gauge->Lecteur + ": ";
          }
        }
      }

      if (bDisplayInfosInStatusBar == true)  // If we have a message to display...
        {
        // Show the message
        if(((Affichage & DISPLAY_STATUS) != 0))  StatusBar->Panels->Items[0]->Text = StatusBar->Panels->Items[0]->Text + " " + txt_suffixe_X_pourcent;
        }
      }
   
delete(potentiometer);

if (Nb_device_chage == true)   // Some driver has been added or removed (USB, firewire...)
  {
  Nb_device_chage = false;

  // ComboBox to select the kind of shading
  Configuration->ComboBox4->Items->Clear();
  for (int i = 0; i < ImageList2-> Count; i++)  Configuration->ComboBox4->Items->Add(i);
  Configuration->ComboBox4->ItemIndex = Type_Texture;
  Configuration->ComboBox4Change(NULL);

  // Test gauge icon
  ImageList1->GetIcon(1, Configuration->Jauge_test->Icone);

 // Radio group: color of the gauges (not used in shading mode)
  if (Texture)  Configuration->ctrl_colors->ItemIndex = 4;
  else
    {
    if (Uniform)  Configuration->ctrl_colors->ItemIndex = 3;
    else  Configuration->ctrl_colors->ItemIndex = NbSeuils;
    }
  // Controls of the main window's behaviour
  Configuration->ctrl_on_top->Checked = (FormStyle == fsStayOnTop);
  Configuration->ctrl_autohide->Checked = bHideMainWindow;
  Configuration->ctrl_hide_dir->ItemIndex = ZoneMasque;
  Configuration->UpDownO1->Position = Timer2->Interval / 1000;

  // Controls for the kinds of drives and settings to monitor
  Configuration->ctrl_memory->Checked = ((Affichage & DISPLAY_MEMORY) != 0);
  Configuration->ctrl_batteries->Checked = ((Affichage & DISPLAY_BATTERY) != 0);
  Configuration->ctrl_gauge_reverse->Checked = InvertDisp;
  Configuration->ctrl_floppy->Checked = ((Affichage & DISPLAY_REMOVABLE_DRV) != 0);
  Configuration->ctrl_cd->Checked = ((Affichage & DISPLAY_CDROM) != 0);
  Configuration->ctrl_hdd->Checked = ((Affichage & DISPLAY_FIXED_DRV) !=  0);
  Configuration->ctrl_network->Checked = ((Affichage & DISPLAY_NET_DRV) != 0);
  Configuration->ctrl_ramdisk->Checked = ((Affichage & DISPLAY_RAMDISK) != 0);
  Configuration->ctrl_main_vol->Checked = ((Affichage & DISPLAY_MAIN_VOL) != 0);
  Configuration->ctrl_wav_vol->Checked = ((Affichage & DISPLAY_WAV_VOL) != 0);
  Configuration->ctrl_midi_vol->Checked = ((Affichage & DISPLAY_MIDI_VOL) != 0);
  Configuration->ctrl_cd_vol->Checked = ((Affichage & DISPLAY_CD_VOL) != 0);
  Configuration->ctrl_aux_vol->Checked = ((Affichage & DISPLAY_AUX_VOL) != 0);
  Configuration->ctrl_mic_vol->Checked = ((Affichage & DISPLAY_MIC_VOL) != 0);
  Configuration->ctrl_status_bar->Checked = ((Affichage & DISPLAY_STATUS) != 0);

  // Thresholds colors
  Configuration->ShapeS0->Brush->Color = Couleur[0];
  Configuration->ShapeS1->Brush->Color = Couleur[1];
  Configuration->ShapeS2->Brush->Color = Couleur[2];

  // Choose the action to perform when double click on a  drive
  Configuration->ctrl_open_mode->ItemIndex = (int)Explorateur;
  Configuration->ctrl_auto_launch->Checked = Autodemarrage;

  // Thresholds numeric settings
  // Caution to not swap theise 3 lines, otherwise the function
  // "DefinitionSeuil" wont be able to set the 3 thresholds
  Configuration->UpDownS3->Position = ValSeuil[2];
  Configuration->UpDownS2->Position = ValSeuil[1];
  Configuration->UpDownS1->Position = ValSeuil[0];

  Configuration->ShapeS3->Brush->Color = Couleur[3];
  Configuration->ShapeUni->Brush->Color = Couleur[4];

  // Normal refresh
  if (Timer1->Interval > 3600000)
    {
    Configuration->UpDown1->Position = Timer1->Interval / 3600000;
    Configuration->ctrl_refresh_unit->ItemIndex = 2;
    }
  else
    {
    if (Timer1->Interval > 60000)
      {
      Configuration->UpDown1->Position = Timer1->Interval / 60000;
      Configuration->ctrl_refresh_unit->ItemIndex = 1;
      }
    else
      {
      Configuration->UpDown1->Position = Timer1->Interval / 1000;
      Configuration->ctrl_refresh_unit->ItemIndex = 0;
      }
    }


  Timer2->Enabled = false;
  Timer1->Enabled = false;

  Configuration->Validation(NULL);
  // Delete old gauges and controls
  CleanUp();
  // Set up new display settings
  Affichage = Configuration->NouvAffichage;
  // Displays new jauges and controls
  DisplaysControls();
  Timer2->Enabled = bHideMainWindow;
  Timer1->Enabled = true;
  }
}


//---------------------------------------------------------------------------

void TPrinc::DropFiles(TMessage &Message)

{


  TPoint pos;

  int nFiles;

  char buffer[256];
             char tabl[255], tabl2[255];;

  nFiles = DragQueryFile((HDROP)Message.WParam, 0xFFFFFFFF, NULL, 0);

  for (int i = 0; i < nFiles; i++)

  {
              for(int i=0;i<=255;i++) buffer[i] ='\0';
                   for(int i=0;i<=255;i++) tabl2[i] ='\0';
     DragQueryFile((HDROP)Message.WParam, i, buffer, 256);

     // Get mouse position when dropping files on the application
     pos = ScreenToClient(Mouse->CursorPos);



  if (GaugeList->Count > 0)
    {
    for(int i = 0; i < GaugeList->Count; i++)
      {
      Gauge = (TJaugeMat*) GaugeList->Items[i];
      if ((Gauge->Lecteur != '@') && (Gauge->Lecteur != '#'))
        {
        if ((Gauge->Top <= pos.y) && (Gauge->Height+Gauge->Top >= pos.y))
          {

          for(int i=0;i<=255;i++) tabl[i] ='\0';
       for(int i=0;i<=255;i++)
       {
          tabl[i]=buffer[i];
       }

      for(int i=0;i<=255;i++)  tabl2[i] =  (AnsiString(Gauge->Lecteur) + AnsiString(":\\") + AnsiString("\0")).c_str()[i];


            SHFILEOPSTRUCT FileOpStr;
          // Copy the dropped file to corresponding drive using windows copy interface
          ZeroMemory(&FileOpStr, sizeof(SHFILEOPSTRUCT));
          FileOpStr.hwnd = Application->Handle;
          FileOpStr.wFunc = FO_COPY;
          FileOpStr.pFrom = tabl;
          FileOpStr.pTo =  tabl2;
          SHFileOperation(&FileOpStr);
          }
        }
      }
    }
  }
  DragFinish((HDROP)Message.WParam);
}

//---------------------------------------------------------------------------

void __fastcall TPrinc::FormCreate(TObject *Sender)
{
  // Background color for icons drawing
  ImageList1->BkColor= clBtnFace;

  // Main window title
  Caption = TXT_VERSION;

  // Hide application's task bar button
  SetWindowLong(Application->Handle, GWL_EXSTYLE, WS_EX_TOOLWINDOW);

  // Avoid system CD request when using GetDiskFreeSpaceEx with a empty drive
  SetErrorMode(SEM_FAILCRITICALERRORS);

  // Create the lists
  GaugeList = new TList;
  SoundCtrlList = new TList;

  // Retrive parameters
  Load_cfg();
  GetIcons();

  // Displays gauges and controls
  DisplaysControls();

  // Make application stay displayed if user press "Show Desktop" icon
  if (FindWindow("Progman", NULL) != NULL)
    {
    ::SetParent(Application->Handle, FindWindow("Progman", NULL));
    }

  // Enable drag and drop
  DragAcceptFiles(Handle, true);
}
//---------------------------------------------------------------------------


